/* OpenVPN - MySQL Logger       *
 *------------------------------*
 * Version 1.0                  *
 * Written by Christoph Haas    *
 * License: LGPL                *
 * 10.06.2010                   *
 */ 

#include <stdio.h>
#include <stdlib.h>

#include <stdarg.h>
#include <mysql/mysql.h>
#include <regex.h>
#include <string.h>

#define __MAX_LINE__   100
#define __MAX_FIELD__   255
#define false      0
#define true      1

MYSQL   mysql;


int escaped_length(char *str)
{
   int   pos;
   int   length;
   int   str_length;

   if(str == NULL)return 4;
   str_length = strlen(str);
   for(pos = 0, length = 0; pos < str_length; pos++, length++)
   {
      switch(str[pos])
      {
         case '\"': length++;
                    break;
         case '\'': length++;
               break;
         case '\\': length += 2;
               break;
         case '\n': length++;
               break;
         case '\r': length++;
               break;
         default  : break;
      }
   }
   return length;
}


int mysql_printf(MYSQL *mysql, char *str, ...)
{
   int    pos_src;
   int    pos_dst;
   int    str_length;
   int    tmp;
   int    this_int;
   int    db_result;
   double    this_double;
   int    length;
   char    tmp_char;
   char   *tmp_str;
   char   *query   = NULL;
   va_list    list;


   str_length   = strlen(str);

   /*   Calculate string length      */

   va_start(list, str);
   for(pos_src = 0, length = 0; pos_src < str_length; pos_src++, length++)
   {
      if(str[pos_src] == '%')
      {
         pos_src++;
         if(pos_src >= str_length)
         {
            fprintf(stderr, "Syntax Error:\t%s\n", str);
            exit(1);
         }
         switch(str[pos_src])
         {
            case '%' : break;
            case 'c' : tmp_char = va_arg(list, int);
                  switch(tmp_char)
                  {
                     case '\'': length++;
                           break;
                     case '\"': length++;
                           break;
                     case '\\': length++;
                           break;
                     case '\n': length++;
                           break;
                     case '\r': length++;
                           break;
                     default  : break;
                  }
                  break;
            case 'd' : this_int = va_arg(list, int);
                  for(tmp = 1; this_int >= 1; tmp++)this_int = this_int/10;
                  length += tmp;
                  break;
            case 'f' : this_double = va_arg(list, double);
                  for(tmp = 20; this_double >= 1; tmp++)this_double = this_double/10;
                  length += tmp;
                  break;
            case 's' : length += (escaped_length(va_arg(list, char *)));
                  break;
            default  : fprintf(stderr, "Syntax Error:\t%s\n", str);
                  break;
         }
      }
   }


   va_end  (list);
   va_start(list, str);



   if((query = malloc((length +1) * sizeof(char))) == NULL)return -1;


   /*   Build valid mysql string   */

   for(pos_src = 0, pos_dst = 0; pos_src < str_length; pos_src++, pos_dst++)
   {
      if(str[pos_src] == '%')
      {
         pos_src++;
         if(pos_src >= str_length)
         {
            fprintf(stderr, "Syntax Error:\t%s\n", str);
            exit(1);
         }
         switch(str[pos_src])
         {
            case '%' : query[pos_dst]   = str[pos_src];
                  break;
            case 'c' : tmp_char = (char)va_arg(list, int);
                  switch(tmp_char)
                  {
                     case '\'': query[pos_dst++]   = '\\';
                           query[pos_dst]   = tmp_char;
                           break;
                     case '\"': query[pos_dst++]   = '\\';
                           query[pos_dst]   = tmp_char;
                           break;
                     case '\\': query[pos_dst++]   = '\\';
                           query[pos_dst]   = tmp_char;
                           break;
                     case '\n': query[pos_dst++]   = '\\';
                           query[pos_dst]   = tmp_char;
                           break;
                     case '\r': query[pos_dst++]   = '\\';
                           query[pos_dst]   = tmp_char;
                           break;
                     default  : query[pos_dst]   = tmp_char;
                           break;
                  }
                  break;
            case 'd' : this_int = va_arg(list, int);
                  query[pos_dst]   = '\0';
                  sprintf(query, "%s%d", query, this_int);
                 /* for(tmp = 0; this_int > 1; tmp++)this_int = this_int/10;
                  pos_dst += tmp;*/

                  pos_dst      = strlen(query)-1;
                  break;
            case 'f' : query[pos_dst]   = '\0';
                  sprintf(query, "%s%f", query, va_arg(list, double));
                  pos_dst      = strlen(query)-1;
                  break;
            case 's' : tmp_str = va_arg(list, char *);
                  if(tmp_str == NULL)
                  {
                     query[pos_dst++] = 'N';
                     query[pos_dst++] = 'U';
                     query[pos_dst++] = 'L';
                     query[pos_dst++] = 'L';
                  }
                  else

                  {
                     for(tmp = 0; tmp < strlen(tmp_str); tmp++)
                     {
                        switch(tmp_str[tmp])
                        {
                           case '\'' : query[pos_dst++] = '\\';
                                  query[pos_dst++] = tmp_str[tmp];
                                  break;
                           case '\"' : query[pos_dst++] = '\\';
                                  query[pos_dst++] = tmp_str[tmp];
                                  break;
                           case '\\' : query[pos_dst++] = '\\';
                                  query[pos_dst++] = tmp_str[tmp];
                                  break;
                           case '\n' : query[pos_dst++] = '\\';
                                  query[pos_dst++] = tmp_str[tmp];
                                  break;
                           case '\r' : query[pos_dst++] = '\\';
                                  query[pos_dst++] = tmp_str[tmp];
                                  break;
                           default   : query[pos_dst++] = tmp_str[tmp];
                                  break;
                        }
                     }
                  }
                  pos_dst--;
                  break;
            default  : fprintf(stderr, "Syntax Error:\t%s\n", str);
                  exit(1);
                  break;
         }
      }
      else query[pos_dst]   = str[pos_src];
   }


   query[pos_dst]   = '\0';
   

   va_end(list);
   db_result = mysql_real_query(mysql, query, strlen(query));
   free(query);



   return db_result;
}




int connect_db(char *config)
{
   FILE   *db_data      = NULL;
   regex_t    reg;
   char    user[__MAX_LINE__]   = "";
   char    passwd[__MAX_LINE__]   = "";
   char    database[__MAX_LINE__]   = "";
   char    host[__MAX_LINE__]   = "";
   int    port         = 0;
   char    tmp[__MAX_LINE__];
   char    buf[__MAX_LINE__];
   int    a, b;
   int    length;

   if((db_data = fopen(config, "r")) == NULL)
   {
      fprintf(stderr, "Can't open %s for reading.\n", config);
      return false;
   }
   regcomp(&reg, "^([a-zA-Z0-9:]+)[ ][a-zA-Z0-9]+$", REG_EXTENDED | REG_NEWLINE);
   while(fgets(buf, __MAX_LINE__, db_data))
   {
      length = strlen(buf);
      if(length < 3)continue;
      if(length >= __MAX_LINE__)
      {
         fclose(db_data);
         regfree(&reg);
         fprintf(stderr, "To long line in config file.\n");
         return false;
      }
      if(buf[0] == '#')continue;
      if(regexec(&reg, buf, 0, 0, 0))
      {
         fclose(db_data);
         regfree(&reg);
         fprintf(stderr, "Syntax error in config file.\n");
         return false;
      }
      for(a = 0, b = 0; a < strlen(buf); a++)
      {
         if(buf[a] == ' ')b++;
      }
      if(b != 1)
      {
         fclose(db_data);
         regfree(&reg);
         fprintf(stderr, "Syntax error in config file.\n");
         return false;
      }
      if((strncmp(buf, "user: ", 6))      == 0)sscanf(buf, "%s %s", tmp, user);
      if((strncmp(buf, "passwd: ", 8))   == 0)sscanf(buf, "%s %s", tmp, passwd);
      if((strncmp(buf, "database: ", 10))   == 0)sscanf(buf, "%s %s", tmp, database);
      if((strncmp(buf, "host: ", 6))      == 0)sscanf(buf, "%s %s", tmp, host);
      if((strncmp(buf, "port: ", 6))      == 0)sscanf(buf, "%s %d", tmp, &port);
   }
   fclose(db_data);
   regfree(&reg);
   if((strlen(user) < 1) || (strlen(passwd) < 1) ||
      (strlen(database) < 1) || (strlen(host) < 1))
   {
      fprintf(stderr, "One value for MySQL connection isn't set. \
Please set user, passwd, database and host.\n");
      return false;
   }




   mysql_init(&mysql);
   if((mysql_real_connect(&mysql, host, user, passwd, database, port, NULL, 0)) == NULL)
   {
      fprintf(stderr, "%s\n", mysql_error(&mysql));
      return false;
   }
   return true;
}



int unlock_table(void)
{
   if(mysql_printf(&mysql, "UNLOCK TABLES"))
   {
      fprintf(stderr, "Can't unlock tables\n%s\n", mysql_error(&mysql));
      mysql_close(&mysql);
      return false;
   }
   mysql_close(&mysql);
   return true;
}



int client_connect(char *config, char *ip, char *local_address,
   char *remote_address, char *client)
{
   MYSQL_RES   *result;


   if(!(connect_db(config)))
   {
      fprintf(stderr, "Can't connect to database.\n");
      return false;
   }


   /****************************************/
   /*   Creating database entrys   */

   /****************************************/

   // Lock tables
   if(mysql_printf(&mysql, "LOCK TABLES `year` WRITE, `month` WRITE, `connections` WRITE"))
   {
      fprintf(stderr, "Can't lock tables\n%s\n", mysql_error(&mysql));
      mysql_close(&mysql);
      return false;
   }





   // Year

   if(mysql_printf(&mysql, "SELECT `id` FROM `year`\
         WHERE `client`='%s' AND `year`=YEAR(CURRENT_DATE) LIMIT 1",
         client))
   {
      fprintf(stderr, "%s\n", mysql_error(&mysql));
      unlock_table();
      return false;
   }
   result = mysql_store_result(&mysql);
   if(mysql_num_rows(result) < 1)
   {
      if(mysql_printf(&mysql, "INSERT INTO `year`\
            (`year`, `client`)VALUES(YEAR(CURRENT_DATE), '%s')",
            client))
      {
         fprintf(stderr, "%s\n", mysql_error(&mysql));
         unlock_table();
         return false;
      }
   }
   mysql_free_result(result);






   // month

   if(mysql_printf(&mysql, "SELECT `id` FROM `month`\
            WHERE   `client`='%s'\
            AND   `year`=YEAR(CURRENT_DATE)\
            AND   `month`=MONTH(CURRENT_DATE)\
            LIMIT   1", client))
   {
      fprintf(stderr, "%s\n", mysql_error(&mysql));
      unlock_table();
      return false;
   }
   result = mysql_store_result(&mysql);
   if(mysql_num_rows(result) < 1)
   {
      if(mysql_printf(&mysql, "INSERT INTO `month` (`month`, `year`, `client`)\
               VALUES(MONTH(CURRENT_DATE), YEAR(CURRENT_DATE), '%s')", client))
      {
         fprintf(stderr, "%s\n", mysql_error(&mysql));
         unlock_table();
         return false;
      }
   }





   // connections

   if(mysql_printf(&mysql, "UPDATE `connections` SET `stop`=UNIX_TIMESTAMP() WHERE\
            `stop`='0' AND `local_ip`='%s' AND `remote_ip`='%s' AND `client`='%s' AND `ip`='%s'",
            local_address, remote_address, client, ip))
   {
      fprintf(stderr, "%s\n", mysql_error(&mysql));
      unlock_table();
      return false;
   }
   if(mysql_printf(&mysql, "INSERT INTO `connections`\
            (`start`, `ip`, `local_ip`, `remote_ip`, `client`)\
            VALUES\
            (UNIX_TIMESTAMP(), '%s', '%s', '%s', '%s')",
            ip, local_address, remote_address, client))
   {
      fprintf(stderr, "%s\n", mysql_error(&mysql));
      unlock_table();
      return false;
   }
   mysql_free_result(result);

   unlock_table();
   return true;
}


int client_disconnect(char *config, char *ip, char *local_address,
      char *remote_address, char *client, char *received, char *sent)
{
   MYSQL_RES   *result;
   MYSQL_ROW    row;
   char       connection_id[__MAX_FIELD__ + 1]   = "";
   char       connect_time[__MAX_FIELD__ + 1]   = "";
   char       disconnect_time[16]         = "";



   if(!(connect_db(config)))
   {
      fprintf(stderr, "Can't connect to database.\n");
      return false;
   }

   // Lock tables

   if(mysql_printf(&mysql, "LOCK TABLES `year` WRITE, `month` WRITE, `connections` WRITE"))
   {
      fprintf(stderr, "Can't lock tables\n%s\n", mysql_error(&mysql));
      mysql_close(&mysql);
      return false;
   }



   // connection ID

   if(mysql_printf(&mysql, "SELECT `id`, UNIX_TIMESTAMP() - `start` AS `connect_time`,\
                            UNIX_TIMESTAMP() AS `timestamp`\
               FROM `connections` WHERE\
                  `ip`='%s'      AND\
                  `local_ip`='%s'      AND\
                  `remote_ip`='%s'   AND\
                  `client`='%s'      AND\
                  `stop`='0'      AND\
                  `sent`='0'      AND\
                  `received`='0'      AND\
                  `start`<UNIX_TIMESTAMP()\
               ORDER BY `id` DESC LIMIT 1",
               ip, local_address, remote_address, client))
   {
      fprintf(stderr, "%s\n", mysql_error(&mysql));
      unlock_table();
      return false;
   }
   result = mysql_store_result(&mysql);
   if(mysql_num_rows(result) > 0)
   {
      row = mysql_fetch_row(result);
      if(row[0] == NULL || row[1] == NULL || row[2] == NULL)
      {
         fprintf(stderr, "Can't get data for connection.\n");
         mysql_free_result(result);
         unlock_table();
         return false;
      }
      strncpy(connection_id, row[0], __MAX_FIELD__);
      connection_id[__MAX_FIELD__] = 0;
      strncpy(connect_time, row[1], __MAX_FIELD__);
      connect_time[__MAX_FIELD__] = 0;
      strncpy(disconnect_time, row[2], 15);
      disconnect_time[15] = 0;
      mysql_free_result(result);
            
      if(mysql_printf(&mysql, "UPDATE `connections` SET\
                     `stop`='%s',\
                     `sent`='%s',\
                     `received`='%s'\
                  WHERE\
                     `ip`='%s'      AND\
                     `local_ip`='%s'      AND\
                     `remote_ip`='%s'   AND\
                     `client`='%s'      AND\
                     `stop`='0'      AND\
                     `sent`='0'      AND\
                     `received`='0'      AND\
                     `id`='%s'      AND\
                     `start`<'%s'\
                  LIMIT 1",
                  disconnect_time, sent, received, ip, local_address, remote_address, client,
                  connection_id, disconnect_time))
      {
         fprintf(stderr, "%s\n", mysql_error(&mysql));
         unlock_table();
         return false;
      }
   }
   else mysql_free_result(result);


   



   // Month

   if(mysql_printf(&mysql, "UPDATE `month` SET\
                  `sent`=`sent`+'%s',\
                  `received`=`received`+'%s',\
                  `time`=`time`+'%s'\
               WHERE\
                  `month`=MONTH(CURRENT_DATE)   AND\
                  `year`=YEAR(CURRENT_DATE)   AND\
                  `client`='%s'\
               LIMIT 1",
               sent, received, connect_time, client))
   {
      fprintf(stderr, "%s\n", mysql_error(&mysql));
      unlock_table();
      return false;
   }



   // Year
   if(mysql_printf(&mysql, "UPDATE `year` SET\
                  `sent`=`sent`+'%s',\
                  `received`=`received`+'%s',\
                  `time`=`time`+'%s'\
               WHERE\
                  `year`=YEAR(CURRENT_DATE)   AND\
                  `client`='%s'\
               LIMIT 1",
               sent, received, connect_time, client))
   {
      fprintf(stderr, "%s\n", mysql_error(&mysql));
      unlock_table();
      return false;
   }



   unlock_table();



   return true;
}



int main(int argc, char **argv)
{
  FILE * pFile;
  pFile = fopen ("logggggg.txt","w");

  fprintf(pFile,"%s %s %s %s %s %s %s",argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]);
   if(argc < 7)
   {
     fprintf(pFile,"argc<7");
      fprintf(stderr,
         "USAGE: %s connect|disconnect configfile IP local_address remote_address \
CommonName recv_bytes sent_bytes\n",
         argv[0]);
      return EXIT_SUCCESS;
   }
   if((strcmp(argv[1], "connect")) == 0)
   {
     fprintf(pFile,"connect");
      client_connect(argv[2], argv[3], argv[4], argv[5], argv[6]);
      fprintf(pFile,"client_connect() finish");
   }

   else if((strcmp(argv[1], "disconnect")) == 0)
   {
     fprintf(pFile,"disconnect");
      if(argc != 9)
      {
	fprintf(pFile,"argc!=0");
         fprintf(stderr,
            "USAGE: %s connect|disconnect configfile IP local_address remote_address \
CommonName recv_bytes sent_bytes\n",
            argv[0]);
      }
      else
      {
	fprintf(pFile,"client_disconnect()");
         client_disconnect(argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8]);
      }
   }
   else

      fprintf(stderr,
         "USAGE: %s connect|disconnect configfile IP local_address remote_address \
CommonName recv_bytes sent_bytes\n",
         argv[0]);


   return EXIT_SUCCESS;
}
